#include "crypeay.h"

#include "cryptopp562\rng.h"
#include "cryptopp562\base64.h"
#include "cryptopp562\queue.h"
#include "cryptopp562\osrng.h"
#include "cryptopp562\files.h"
#include "cryptopp562\aes.h"
#include "cryptopp562\filters.h"
#include "cryptopp562\modes.h"


#define AES_BLOCK_SIZE 16

using namespace CryptoPP;


#pragma comment(lib, "cryptlib.lib")


#define AES_128  128;
#define AES_256  256;

extern "C" CRYPEAY_API int CrypEay_AES(const unsigned char* pIn, int nInLen, const char* pKey, std::string& strOut){
	//PKCS#5
	int nBei = nInLen / AES_BLOCK_SIZE + 1;
	int nTotal = nBei * AES_BLOCK_SIZE;
	char *enc_s = (char*)malloc(nTotal+1);
	int nNumber;
	if (nInLen % 16 > 0){
		nNumber = nTotal - nInLen;
	}
	else{
		nNumber = 16;
	}
	memset(enc_s, 0, nTotal + 1);
	memset(enc_s, nNumber, nTotal);
	memcpy(enc_s, pIn, nInLen);


	//Կѡaes128aes256
	int nKeyLen = strlen(pKey);

	int nEncryptMode;
	if (nKeyLen == 16){
		nEncryptMode = AES_128;
	}
	else if (nKeyLen == 32){
		nEncryptMode = AES_256;
	}
	else{
		SetErrMsg(g_errmsg, "invalid aes encrypt mode");
		free(enc_s);
		return CRYPEAY_INVALIDPARAM;
	}


	//aes
	string cipher;
	bool bEncrypt = true;
	try{
		ECB_Mode<AES>::Encryption ecbAes;
		
		if (nEncryptMode == 128){
			ecbAes.SetKey((const byte*)pKey, 16);
		}
		else if (nEncryptMode == 256){
			ecbAes.SetKey((const byte*)pKey, 32);
		}

		StringSource ss1(enc_s, true,
		new StreamTransformationFilter(ecbAes,
		new StringSink(cipher),
		StreamTransformationFilter::BlockPaddingScheme::NO_PADDING
		) // StreamTransformationFilter      
		); // StringSource

	}
	catch (CryptoPP::Exception& e){
		bEncrypt = false;
	}

	if (!bEncrypt){
		free(enc_s);
		return CRYPEAY_ENCRYPTFAIL;
	}
	//base64
	Base64Encoder encoder;
	encoder.Put((const byte*)cipher.c_str(), cipher.length());
	encoder.MessageEnd();
	word64 wSize = encoder.MaxRetrievable();

	if (wSize){
		strOut.resize(wSize);
		encoder.Get((byte*)strOut.data(), strOut.size());
	}
	free(enc_s);
	return CRYPEAY_OK;
}


int main(){
	string tmp;
	//const unsigned char* plaintxt = (const unsigned char*)"adfsdfwerwerwerwer";

	string plaintext = "this is a test!";
	CrypEay_AES((const unsigned char*)plaintext.c_str(), plaintext.length(), "!!king?DeE|12345", tmp);
	return 0;
}